import numpy as np
import tifffile as tif
import sys
import os
from scipy.ndimage.filters import gaussian_filter as gaussfil
from scipy.ndimage import imread as ndimread
from astropy.io import fits
import alipy
"""
    Un "cubo" e' un array multidimensionale del tipo:
       cubo[N][H][L][C]
    Il primo incice [N] identifica l'immagine nella sequenza.
    Il secondo [H] la riga dell'immagine (altezza).
    Il terzo [L] la colonna dell'immagine (larghezza).
    Il quarto [C, se presente] il canale di colore.
"""


def calcMediana(cubo,salvamem=False):
  """ Calcola la mediana di una sequenza di immagini (cubo);
      restituisce in uscita un'immagine simile a quelle che
      compongono la sequenza, con forma (H,L,C).
      Se "salvamem" e' vero, risparmia memoria operando sul
      cubo in ingresso, ma distruggendone i dati.
  """
  if(type(cubo) != np.ndarray):
    sys.stderr.write("errore: non np.array\n")
    return False
  return np.median(cubo,axis=0,overwrite_input=salvamem).astype(cubo.dtype)



def calcMedia(cubo):
  """ Calcola la media di una sequenza di immagini (cubo);
      restituisce in uscita un'immagine simile a quelle che
      compongono la sequenza, con forma (H,L,C).
  """
  if(type(cubo) != np.ndarray):
    sys.stderr.write("errore: non np.array\n")
    return False
  return np.mean(cubo,axis=0).astype(cubo.dtype)



def calcMediaFloat(cubo):
  """ Calcola la media di una sequenza di immagini (cubo);
      restituisce in uscita un'immagine simile a quelle che
      compongono la sequenza, con forma (H,L,C).
  """
  if(type(cubo) != np.ndarray):
    sys.stderr.write("errore: non np.array\n")
    return False
  return np.mean(cubo,axis=0)



def calcSigma(cubo):
  """ Calcola la deviazione standard sul primo asse. In uscita una
      immagine simile a quelle della sequenza.
  """
  if(type(cubo) != np.ndarray):
    sys.stderr.write("errore: non np.array\n")
    return False
  n = cubo.shape[0]
  mu = cubo.mean(axis=0)
  std = 0
  for i in range(n):
    diff = (cubo[i]-mu)
    diff *= diff
    std += diff
  return np.sqrt(std/n)

def calcSigmaRef(cubo, ref):
  """ Calcola la deviazione standard sul primo asse rispetto ad una
      immagine di riferimento. In uscita un' immagine simile a quelle
      della sequenza.
  """
  if(type(cubo) != np.ndarray):
    sys.stderr.write("errore: non np.array\n")
    return False
  if(type(ref) != np.ndarray):
    sys.stderr.write("errore: non np.array\n")
    return False
  n = cubo.shape[0]
  std = 0
  for i in range(n):
    diff = (cubo[i]-ref)
    diff *= diff
    std += diff
  return np.sqrt(std/n)


def seqSigClip(cubo,kappa,media=False):
  """ Individua i valori da trascurare secondo il criterio
      del sigma clip. "kappa" e' il coefficiente cui va
      moltiplicata la deviazione standard. Il taglio viene
      effettuato prendendo la mediana come valore centrale,
      amenoche' media non sia "True" e allora usa la media.
      Restituisce una maschera (cubo di bool).
  """
  n = cubo.shape[0]
  if(media):
    centro = calcMedia(cubo)
  else:
    centro = calcMediana(cubo)
  sigma = calcSigma(cubo)
  mask0 = ((kappa*sigma-np.absolute(np.float128(cubo[0])-centro))>=0)
  mask0 = mask0[None,...]
  for i in range(n)[1:]:
    maskimg = ((kappa*sigma-np.absolute(np.float128(cubo[i])-centro))>=0)
    maskimg = maskimg[None,...]
    mask0 = np.concatenate((mask0,maskimg))
  return mask0



def calcSigClip(cubo, kappa, media=False, debug=False):
  """ Passa gli argomenti a seqSigClip. Dopo esegue la media aritmetica.
      In uscita un'immagine simile a quelle della sequenza, ma se dedug
      e' abilitato fornisce anche un array con i valori considerati come
      validi per ciascun pixel.
  """
  mask = seqSigClip(cubo,kappa,media)
  if debug:
    return (cubo*mask).sum(axis=0)/(mask.sum(axis=0)),mask.sum(axis=0)
  else:
    return (cubo*mask).sum(axis=0)/(mask.sum(axis=0))



def genLista(nomefile):
  """ Genera una lista (array normale) contenente i nomi dei
      file da aprire, ricevendo in ingresso il nome del file
      che contiene tali nomi.
  """
  try:
    lista = [riga.rstrip('\n') for riga in open(str(nomefile))]
  except:
    sys.stderr.write("errore input:"+str(nomefile)+"\n")
    return False
  return lista



def genCubo(lista,mmap=False):
  """ Genera il "cubo" (sequenza di immagini). Accetta in
      ingresso un array con i nomi dei file oppure una
      stringa con il nome del file di testo che contiene i
      nomi dei file da caricare. L'argomento opzionale
      mmap permette di non caricare effettivamente i dati
      in memoria ma di leggerli dal disco (piu' lento ma
      utile se ci sono tante immagini).
  """
  if(type(lista) == str):
    lista = genLista(lista)
  num = len(lista)
  try:
    im0 = tif.imread(lista[0],memmap=mmap)
  except:
    sys.stderr.write("errore input:"+lista[0]+"\n")
    return False
  ishape = im0.shape
  idtype = im0.dtype
  im0 = im0[None,...]

  for i,x in enumerate(lista[1:]):
    try:
      imi = tif.imread(x,memmap=mmap)
      if (imi.shape != ishape):
        sys.stderr.write("errore dimensioni:"+x+"\n")
        return False
      imi = imi[None,...]
      im0 = np.concatenate((im0,imi))
    except:
      sys.stderr.write("errore input:"+x+"\n")
      return False
  return im0


def genCuboND(lista,mmap=False):
  """ Genera il "cubo" (sequenza di immagini). Accetta in
      ingresso un array con i nomi dei file oppure una
      stringa con il nome del file di testo che contiene i
      nomi dei file da caricare. L'argomento opzionale
      mmap permette di non caricare effettivamente i dati
      in memoria ma di leggerli dal disco (piu' lento ma
      utile se ci sono tante immagini). Usa la libreria
      scipy.ndimage che accetta anche png, ma non funzione
      con i 16bit.
  """
  if(type(lista) == str):
    lista = genLista(lista)
  num = len(lista)
  try:
    im0 = ndimread(lista[0])
  except:
    sys.stderr.write("errore input:"+lista[0]+"\n")
    return False
  ishape = im0.shape
  idtype = im0.dtype
  im0 = im0[None,...]

  for i,x in enumerate(lista[1:]):
    try:
      imi = ndimread(x)
      if (imi.shape != ishape):
        sys.stderr.write("errore dimensioni:"+x+"\n")
        return False
      imi = imi[None,...]
      im0 = np.concatenate((im0,imi))
    except:
      sys.stderr.write("errore input:"+x+"\n")
      return False
  return im0


def scompColori(cubo):
  """ Separa un cubo in tre cubi monocromatici: in uscita
      tre sequenze di dimensione (N,H,L), senza il quarto
      asse per il colore.
  """
  if(type(cubo) != np.ndarray):
    sys.stderr.write("errore: non np.array\n")
    return False
  return cubo[...,0], cubo[...,1], cubo[...,2]



def imgBiancoNero(img,somma=0):
  """ Produce un'immagine monocromatica. Pesa ciascun canale
      allo stesso modo (1,1,1). In uscita un immagine del tipo
      (H,L). Se somma e' vero, non divide per 3.
  """
  if((len(img.shape) != 3) or (img.shape[2]!=3)):
    sys.stderr.wrtite("errore.\n")
    return False
  r,g,b = scompColori(img)
  media_r = media_g = media_b = 1.0
  if(somma):
    return np.clip((r.astype(np.float128)*media_r+g.astype(np.float128)*media_g+b.astype(np.float128)*media_b),0,np.iinfo(img.dtype).max).astype(img.dtype)
  else:
    return np.clip((r.astype(np.float128)*media_r+g.astype(np.float128)*media_g+b.astype(np.float128)*media_b)/(media_r+media_g+media_b),0,np.iinfo(img.dtype).max).astype(img.dtype)



def seqFlatUSM(cubo,dark):
  """ Normalizza una sequenza di immagini per fare un flat. 
      Riceve in ingresso un cubo, un bias e un dark dei flat.
      Rimuove bias e dark. Passa in bianco e nero. Stima un
      valore di luminosita' centrale media per ciascuna immagine
      convolvendole con una funzione gaussiana larga e prendendo
      il massimo. Successivamente, rinormalizza le immagini in
      modo che tale valore sia pari, per ciascuna, alla media di
      tutti i valori.
      In uscita un cubo delle stesse dimensioni monocromatico.
  """
  n  = cubo.shape[0]
  ly = cubo.shape[1]
  lx = cubo.shape[2]
  rgaus = ((1.0*lx*ly)**0.5)/40.0
  cudim = cubo.shape[0:3]
  monocubo = np.zeros(cudim,cubo.dtype)
  lnormval  = np.ones(n,cubo.dtype)
  for i in range(n):
    cubo[i] = imgSottrai(cubo[i],dark)
    monocubo[i] = imgBiancoNero(cubo[i],somma=1)
    lnormval[i] = np.float128(np.amax(gaussfil(monocubo[i], sigma=rgaus)))
  normval = np.average(lnormval)
  for i in range(n):  
    monocubo[i] = imgMoltiplica(monocubo[i],normval/lnormval[i])
  return monocubo

    

def seqRiduci(cubo,bias,dark,flat,darkesp,esposizioni):
  """ Riduci la sequenza cubo, sottraendo dark e bias e 
      dividendo per il flat field. In input:
       - cubo: il cubo di immagini;
       - bias: l'immagine masterbias;
       - dark: l'immagine masterdark;
       - flat: l'immagine masterflat;
       - darkesp: tempo di esposizione del masterdark;
       - esposizioni: array numpy 1d con i tempi di posa
         delle immagini nel cubo.
  """
  if(type(cubo) != np.ndarray):
    sys.stderr.write("errore: non np.array\n")
    return False
  if(type(bias) != np.ndarray):
    sys.stderr.write("errore: non np.array\n")
    return False
  if(type(dark) != np.ndarray):
    sys.stderr.write("errore: non np.array\n")
    return False
  if(type(flat) != np.ndarray):
    sys.stderr.write("errore: non np.array\n")
    return False
  if(type(esposizioni) != np.ndarray):
    sys.stderr.write("errore: non np.array\n")
    return False
  n = cubo.shape[0]
  Ccolori = (len(cubo.shape) == 4)
  Fcolori = (len(flat.shape) == 3)
  if Ccolori:
    print "elaboro a colori"
    if (not Fcolori):
      triflat = np.zeros(flat.shape+(3,),dtype=flat.dtype)
      triflat[:,:,0] = flat
      triflat[:,:,1] = flat
      triflat[:,:,2] = flat
    else:
      triflat = flat
    cdim = cubo.shape[1:3]
    bdim = bias.shape[0:2]
    ddim = dark.shape[0:2]
    fdim = triflat.shape[0:2]
  else:
    print "elaboro in bianco e nero"
    triflat = flat
    cdim = cubo.shape[1:]
    bdim = bias.shape
    ddim = dark.shape
    fdim = flat.shape
  
  if ((cdim != bdim) or (cdim != ddim) or (cdim != fdim)):
    sys.stderr.write("errore dimensioni\n")
    return False

  costanti = (np.float128(esposizioni)/np.float128(darkesp)).astype(np.float128)
  for i in range(n):
    cubo[i] = (np.clip(np.float128(np.float128(cubo[i]) - bias - costanti[i]*dark)/(np.float128(triflat)/(np.iinfo(triflat.dtype).max)),0,np.iinfo(cubo.dtype).max)).astype(cubo.dtype)



def imgSomma(img1,img2):
  """ Funzioni per operazioni aritmetiche sulle immagini. La  
    dinamica delle immagini, dopo le operazioni aritmetiche 
    eseguite in precisione float128, viene tagliata entro i
    limiti della dinamica originaria (0,max). Accetta anche
    costanti.
  """
  if(type(img1) == np.ndarray):
    bim = True
    if(type(img2) == np.ndarray):
      if(img1.shape != img2.shape):
        sys.stdout.write("errore dimensioni.\n")
        return False
  elif(type(img2) == np.ndarray):
    bim = False
  else:
    sys.stdout.write("errore: non np.array\n")
  return np.clip(np.float128(img1) + np.float128(img2),0,np.iinfo(img1.dtype).max).astype(img1.dtype)



def imgSottrai(img1,img2):
  """ Funzioni per operazioni aritmetiche sulle immagini. La  
    dinamica delle immagini, dopo le operazioni aritmetiche 
    eseguite in precisione float128, viene tagliata entro i
    limiti della dinamica originaria (0,max). Accetta anche
    costanti.
  """
  if(type(img1) == np.ndarray):
    bim = True
    if(type(img2) == np.ndarray):
      if(img1.shape != img2.shape):
        sys.stdout.write("errore dimensioni.\n")
        return False
  elif(type(img2) == np.ndarray):
    bim = False
  else:
    sys.stdout.write("errore: non np.array\n")
  return np.clip(np.float128(img1) - np.float128(img2),0,np.iinfo(img1.dtype).max).astype(img1.dtype)



def imgMoltiplica(img1,img2):
  """ Funzioni per operazioni aritmetiche sulle immagini. La  
    dinamica delle immagini, dopo le operazioni aritmetiche 
    eseguite in precisione float128, viene tagliata entro i
    limiti della dinamica originaria (0,max). Accetta anche
    costanti.
  """
  if(type(img1) == np.ndarray):
    bim = True
    if(type(img2) == np.ndarray):
      if(img1.shape != img2.shape):
        sys.stdout.write("errore dimensioni.\n")
        return False
  elif(type(img2) == np.ndarray):
    bim = False
  else:
    sys.stdout.write("errore: non np.array\n")
  return np.clip(np.float128(img1) * np.float128(img2),0,np.iinfo((img1 if bim else img2).dtype).max).astype((img1 if bim else img2).dtype)



def imgDividi(img1,img2):
  """ Funzioni per operazioni aritmetiche sulle immagini. La  
    dinamica delle immagini, dopo le operazioni aritmetiche 
    eseguite in precisione float128, viene tagliata entro i
    limiti della dinamica originaria (0,max).  Accetta anche
    costanti.
  """
  if(type(img1) == np.ndarray):
    bim = True
    if(type(img2) == np.ndarray):
      if(img1.shape != img2.shape):
        sys.stdout.write("errore dimensioni.\n")
        return False
  elif(type(img2) == np.ndarray):
    bim = False
  else:
    sys.stdout.write("errore: non np.array\n")
  return np.clip(np.float128(img1) / np.float128(img2),0,np.iinfo((img1 if bim else img2).dtype).max).astype((img1 if bim else img2).dtype)



""" Importo "imread" e "imsave" da tifffile cosi' come sono."""
imgCarica = tif.imread
imgSalva  = tif.imsave



def imgScalaLog(img, k):
  """ Riscalamento logaritmico dei valori dei pixel. La mappa 
      utilizzata e':
                 Y = M * log(k*x/M +1)/log(k+1)
      dove M e' il massimo della dinamica e k e' un parametro
      che definisce quanto i valori bassi vengono espansi e 
      quelli alti compressi.
  """
  if(type(img) != np.ndarray):
    sys.stderr.write("errore: non np.array\n")
    return False
  imglog = np.clip(np.iinfo(img.dtype).max * np.log(k * np.float128(img)/(np.iinfo(img.dtype).max) + 1)/np.log(np.float128(k)+1),0,np.iinfo(img.dtype).max).astype(img.dtype)
  return imglog


def imgCompBassi(img, a):
  """ Comprimi i livelli bassi di un'immagine con una funzione
      liscia ( ((x>0)*(x<1)*tanh(tan(pi/2.0/a*x))+(x>=1))*x ).
      Il taglio viene effettuato sotto il livello "a".
  """
  return (((img>0)*(img<a)*np.tanh(np.tan(3.141592/2.0*img/a)) + (img>=a))*img).astype(img.dtype)


def imgSalvaFit(nomefile, img, flip=True):
  """ Salva l'immagine sotto forma di file FITS per eseguirci 
      poi sopra SEXTRACTOR per l'allineamento. Se flip e' vero,
      la ribalta prima di salvarla.
  """
  if(type(img) != np.ndarray):
    sys.stderr.write("errore: non np.array\n")
    return False
  if(flip):
    hdu = fits.PrimaryHDU(np.uint16(np.flipud(img)),uint=True)
  else:
    hdu = fits.PrimaryHDU(np.uint16(img),uint=True)
  hdu.writeto(nomefile,clobber=True)



def imgCaricaFit(nomefile,flip=True):
  """ Carica un file fit come immagine. Se flip e' vero,
      la ribalta una volta caricata.
  """
  try:
    im0 = fits.getdata(nomefile,uint=True)
  except:
    sys.stderr.write("errore input:"+nomefile+"\n")
    return False
  if(im0.dtype != np.uint16):
    minimo  = np.amin(im0)
    massimo = np.amax(im0)
    intervallo = massimo-minimo
    im0 = np.clip((np.float128(im0) - minimo)/intervallo*np.iinfo(np.uint16).max,0,np.iinfo(np.uint16).max).astype(np.uint16)
  if(flip):
    return np.flipud(im0)
  else:
    return im0



def seqSalva(prefisso, cubo):
  """ Salva un cubo di immagini come file TIFF, specificando  
      un prefisso incrementale.
  """
  if(type(cubo) != np.ndarray):
    sys.stderr.write("errore: non np.array\n")
    return False
  n = cubo.shape[0]
  for i in range(n):
    imgSalva((prefisso+str(i)+'.TIF'),cubo[i])



def seqSalvaFit(prefisso, cubo,flip=True):
  """ Salva un cubo di immagini come file FITS, specificando  
      un prefisso incrementale.
  """
  if(type(cubo) != np.ndarray):
    sys.stderr.write("errore: non np.array\n")
    return False
  n = cubo.shape[0]
  for i in range(n):
    imgSalvaFit((prefisso+str(i)+'.FIT'),cubo[i],flip)



def allineaFit(lista, listaG, listaB, referenza=0):
  """ Allinea le immagini con alipy e SEXTRACTOR. Fornire una 
      lista con i nomi delle immagini FITS del canale rosso,
      una del canale verde con referenza la stessa del rosso 
      e una per il blu. Viene preso il rosso come referenza,
      e il numero dell'immagine puo' essere specificato
      attraverso "referenza".
  """
  if(type(lista) == str):
    lista = genLista(lista)
  if(type(listaG) == str):
    listaG = genLista(listaG)
  if(type(listaB) == str):
    listaB = genLista(listaB) 
  if(referenza == 0):
    referenza = lista[0]
  outputshape = alipy.align.shape(referenza)
  ident = alipy.ident.run(referenza, lista, visu=False)
  print "========================== RIEPILOGO =========================="
  for idn in ident:
    if idn.ok == True:
      print "%3s : %3s, flusso %.2f" % (idn.ukn.name, idn.trans, idn.medfluxratio)
    else:
      print "%10s : non trovato !" % (idn.ukn.name)
  for j,idn in enumerate(ident):
    if idn.ok == True:
      alipy.align.affineremap(idn.ukn.filepath, idn.trans, shape=outputshape, makepng=False)
      alipy.align.affineremap(listaG[j], idn.trans, shape=outputshape, makepng=False)
      alipy.align.affineremap(listaB[j], idn.trans, shape=outputshape, makepng=False)
    else:
      sys.stderr.write("errore allineamento: "+str(idn.ukn.filepath)+"\n")



def allineaCubo(cubo, numref=0, elimina=True):
  """ Allinea un cubo, producendo in uscita un cubo uguale
      ma allineato. "numref" viene passato a allineaFit. Se
      "elimina" e' falso, conserva tutti i file temporanei
      creati da alipy e sextractor.
  """
  n = cubo.shape[0]
  origdtype = cubo.dtype
  if(numref >= n):
    sys.stderr.write("numref = "+str(numref)+" : valore fuori dal range\n")
    return False
  r,g,b = scompColori(cubo)
  del cubo
  seqSalvaFit("r",r,flip=0)
  seqSalvaFit("g",g,flip=0)
  seqSalvaFit("b",b,flip=0)
  listar = [ "r"+str(i)+".FIT" for i in range(n)]
  listag = [ "g"+str(i)+".FIT" for i in range(n)]
  listab = [ "b"+str(i)+".FIT" for i in range(n)]
  referenza = listar[numref]
  allineaFit(listar,listag,listab,referenza)
  del r,g,b
  cartella = os.getcwd()
  nuova = cartella+"/alipy_out"
  os.chdir(nuova)
  listarn = [ "r"+str(i)+"_affineremap.fits" for i in range(n)]
  listagn = [ "g"+str(i)+"_affineremap.fits" for i in range(n)]
  listabn = [ "b"+str(i)+"_affineremap.fits" for i in range(n)]
  r = genCuboFit(listarn,flip=0)
  g = genCuboFit(listagn,flip=0)
  b = genCuboFit(listabn,flip=0)
  if(elimina):
    listan = listarn + listagn + listabn
    for i in listan:
      os.system("rm "+str(i))
    os.chdir(cartella)
    lista = listar + listag + listab
    for i in lista:
      os.system("rm "+str(i))
    os.system("rmdir alipy_out")
  os.chdir(cartella)
  return combinaColori(r,g,b).astype(origdtype)
  
  

def combinaColori(r,g,b):
  """ Combina tre cubi monocromatici (o immagini) in un unico
      cubo policromatico. Se si parte da 3x(N,H,L) si finisce
      con (N,H,L,C); se si parte da 3x(H,L) in (H,L,C).
  """
  if((type(r) != np.ndarray) or (type(g) != np.ndarray) or (type(b) != np.ndarray)):
    sys.stderr.write("errore: non np.array\n")
  dimr = r.shape
  dimg = g.shape
  dimb = b.shape
  if((dimr != dimg) or (dimr != dimb) or (dimg != dimb)):
    sys.stderr.write("errore dimensioni\n")
    return False
  r = r[...,None]
  g = g[...,None]
  b = b[...,None]
  
  r = np.concatenate((r,g,b),axis=len(dimr))
  return r


def genCuboFit(lista,flip=1):
  """ Genera un cubo a partire da dei file fits.              """
  if(type(lista) == str):
    lista = genLista(lista)
  num = len(lista)
  try:
    im0 = fits.getdata(lista[0])
  except:
    sys.stderr.write("errore input:"+lista[0]+"\n")
    return False
  ishape = im0.shape
  idtype = im0.dtype
  cshape = (num,)+ishape
  
  im0 = im0[None,...]

  cubo = np.zeros(cshape,idtype)
  for i in range(num)[1:]:
    try:
      imi = imgCaricaFit(lista[i],flip)
      if (imi.shape != ishape):
        sys.stderr.write("errore dimensioni:"+x+"\n")
        return False
      imi = imi[None,...]
      im0 = np.concatenate((im0,imi))
    except:
      sys.stderr.write("errore input:"+x+"\n")
      return False
  return im0


def bin2x2Media(cubo):
  """ Esegue il binning 2x2 con la media di un'immagine o un
      cubo. In uscita un cubo o immagine in formato float128,
      che puo' essere ridotto al formato originale oppure
      espanso ad uno superiore.
  """
  if(len(cubo.shape)<4):
    cubo = cubo[None,...]
  ly = cubo.shape[1]
  lx = cubo.shape[2]
  if(ly%2 != 0):
    cubo = cubo[:,0:ly-1,...]
    ly = cubo.shape[1]
  if(lx%2 != 0):
    cubo = cubo[:,:,0:lx-1,...]
    lx = cubo.shape[2]
  nlx = lx/2
  nly = ly/2
  bincubo = (cubo[0].astype(np.float128)).reshape(nly,2,nlx,2,-1).mean(axis=(1,3))
  bincubo = bincubo[None,:]
  for img in cubo[1:,...]:
    timg = (img.astype(np.float128)).reshape(nly,2,nlx,2,-1).mean(axis=(1,3))
    timg = timg[None,:]
    bincubo = np.concatenate((bincubo,timg))
  if(bincubo.shape[0]==1):
    return bincubo[0]
  else:
    return bincubo


def bin3x3Media(cubo):
  """ Esegue il binning 3x3 con la media di un'immagine o un
      cubo. In uscita un cubo o immagine in formato float128,
      che puo' essere ridotto al formato originale oppure
      espanso ad uno superiore.
  """
  if(len(cubo.shape)<4):
    cubo = cubo[None,...]
  ly = cubo.shape[1]
  lx = cubo.shape[2]
  if(ly%3 != 0):
    cubo = cubo[:,0:ly-1,...]
    ly = cubo.shape[1]
    if(ly%3 != 0):
      cubo = cubo[:,0:ly-1,...]
      ly = cubo.shape[1]
  if(lx%2 != 0):
    cubo = cubo[:,:,0:lx-1,...]
    lx = cubo.shape[2]
    if(lx%2 != 0):
      cubo = cubo[:,:,0:lx-1,...]
      lx = cubo.shape[2]
  nlx = lx/3
  nly = ly/3
  bincubo = (cubo[0].astype(np.float128)).reshape(nly,3,nlx,3,-1).mean(axis=(1,3))
  bincubo = bincubo[None,:]
  for img in cubo[1:,...]:
    timg = (img.astype(np.float128)).reshape(nly,3,nlx,3,-1).mean(axis=(1,3))
    timg = timg[None,:]
    bincubo = np.concatenate((bincubo,timg))
  if(bincubo.shape[0]==1):
    return bincubo[0]
  else:
    return bincubo


def seqStatInfo(cubo):
  """ Stampa le statistiche delle singole
      immagini di un cubo. Calcola il
      minimo delle immagini convolvendole
      prima con una gaussiana con sigma=
      20px.
  """
  sys.stdout.write("# Numero\t Media\t Varianza\tGaussMin\n")
  for i,img in enumerate(cubo):
    sfu = gaussfil(img,sigma=20)
    sys.stdout.write(str(i)+"\t"+str(np.mean(img))+"\t"+str(np.std(img))+"\t"+str(np.amin(sfu))+"\n")
    


def infoRoba(obj):
  """ Usami se ti dimentichi chi e' cosa."""
  print " ",type(obj)
  if(type(obj)==np.ndarray):
    print "  dimensioni:",str(obj.shape)
    print "  tipo dati: ",str(obj.dtype)
  elif((type(obj)==tuple) or (type(obj)==list)):
    print "  lunghezza:",len(obj)
    tipo = type(obj[0])
    if(all(isinstance(i, tipo) for i in obj)):
      print "  tipo dati:",type(obj[0])
    else:
      print "  tipo dati: non omogeneo",
      


def infoFunzioni():
  """Una lista delle funzioni"""
  print """  *  genLista("lista.txt")\t\t\t\t list

  *  genCubo(lista)\t\t\t\t\t cubo
     genCubo("lista.txt")\t\t\t\t cubo
  *  genCuboFit(lista)\t\t\t\t\t cubo
     genCuboFit("lista.txt")\t\t\t\t cubo

  *  calcMediana(cubo)\t\t\t\t\t img
  *  calcMedia(cubo)\t\t\t\t\t img
  *  calcSigClip(...)\t\t\t\t\t img

  *  scompColori(cubo)\t\t\t\t\t cubo,cubo,cubo
  *  combinaColori(cubor,cubog,cubob)\t\t\t cubo

  *  seqNormFlatGauss(cubo)\t\t\t\t -

  *  seqRiduci(cubo,bias,dark,flat,darkesp,esposizioni)\t -

  *  imgSomma(img1,img2)\t\t\t\t img
     imgSomma(img,costante)\t\t\t\t img
     imgSomma(costante,img)\t\t\t\t img
  *  imgSottrai(img1,img2 | costante)\t\t\t img
  *  imgMoltiplica(img1,img2 | costante)\t\t img
  *  imgDividi(img1,img2 | costante)\t\t\t img

  *  imgCarica = tif.imread\t\t\t\t img
  *  imgSalva = tif.imsave\t\t\t\t -
  *  seqSalva("prefisso", cubo)\t\t\t\t -

  *  imgCaricaFit(nomefile): \t\t\t\t img
  *  imgSalvaFit(nome,img,flip=1) \t\t\t -
  *  seqSalvaFit(prefisso,cubo,flip=1)\t\t\t -

  *  imgScalaLog(img,k)\t\t\t\t\t img

  *  allineaFit(listaR, listaG, listaB)\t\t\t -
     allineaFit("listaR.txt","listaG.txt","listaB.txt")\t -
     allineaFit(liste, "referenza.fit")\t\t\t -
  *  allineaCubo(cubo, numref=0, elimina=True)\t\t cubo

  *  bin2x2Media(img)\t\t\t\t\t img

  *  infoRoba(obj)\t\t\t\t\t -

  *  infoFunzioni()\t\t\t\t\t -
  """

